//--------------------------------------------------------------------------
//
// Copyright 2008, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international
// treaty provisions. Cypress hereby grants to licensee a personal,
// non-exclusive, non-transferable license to copy, use, modify, create
// derivative works of, and compile the Cypress Source Code and derivative
// works for the sole purpose of creating custom software in support of
// licensee product to be used only in conjunction with a Cypress integrated
// circuit as specified in the applicable agreement. Any reproduction,
// modification, translation, compilation, or representation of this
// software except as specified above is prohibited without the express
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED,
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress' product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//--------------------------------------------------------------------------
//*****************************************************************************
//*****************************************************************************
//  FILENAME: main.c
//   Version: 1.0, Updated on 21 November 2008
//
//  DESCRIPTION: Main file of the Example_EEPROM_24C256 Project.
//
//-----------------------------------------------------------------------------
//  Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************
//*****************************************************************************
// 
//Project Objective
//	    To demonstrate the interfacing of I2C based temperature sensor LM75 with PSoC.
//
//Overview
//	This project is designed to demonstrate the interfacing of I2C Temperature Sensor with PSoC.
//	In this I2C protocol is used to control the transfer between the sensor and PSoC.
//
//	The following changes were made to the default settings in the Device Editor:
// 
//    	Select User Modules
//    	    o Select I2CHW_1, LCD_1, Timer16_1 user module. 
//    	    o In this example, these UM is renamed as I2CHW, LCD and OneSecTimer 
//    	Place User Modules
//    	    1. The User Module I2CHW will occupy the space in deicated system resources.
//			2. LCD user module is implemented completely in the software.
//			3. Timer16 user mdoule is placed in DBB00 and DBB01.
//
//    	Set the global resources and UM parameters in the Device Editor as shown under 
//    	"Project Settings" ahead.
//
//	System will use the I2CHW module for any communication with the temperature sensor. The 
//  protocol is implemented in the hardware and High-Level APIs are used to control the transfer.
//	Timer16 is used to generate an interrupt after every 1 second to initiate the read temperature
//  from the sensor and display it on LCD.
//
// Operation
//	Upon program execution all hardware settings from the device configuration are loaded 
//	into the device and main.c is executed. After every 1 second an interrupt will be genrated 
//	on which temperature will be read from the Temperature Sensor and is updated on the LCD.
//
// Circuit Connections
// 	This example can be tested using the CY3210 PSoC Eval1 board.  The following connections
// 	are to be made.  
// 
// 	PSoC
// 	--------
// 		P1[5] - SDA 			(Externally pull up to Vcc using 2.2K resisgtor)
// 		P1[7] - SCL 			(Externally pull up to Vcc using 2.2K resisgtor)
// 	
// 	LM75C (8 Pin SOIC) - Temperature Sensor		
// 	---------------------------
// 		Pin 1		- SDA		Connect to P1[5] of PSoC
// 		Pin 2		- SCL		Connect to P1[7] of PSoC
//		Pin 3		- OS		No Connect
// 		Pin 4		- Ground
// 		Pin 5,6,7	- Ground    (to make the 7 bit address as 1001 000)
// 		Pin 8		- Vcc.	
//
//Project Settings
//
//    Global Resources 
//        CPU_Clock      = SysClk/2      		CPU clock set to 12 MHz
//		  VC1            = 10
//		  VC3 Source     = VC1
//		  VC3 Divider	 = 240					To generate a signal of 10KHz
//
//User Module Parameters
//    I2CHW		
//        Read_Buffer_Types   	  	= RAM ONLY	    	It can access only RAM buffer for transfer of data.
//        CPU_Clk_speed_(CY8C27xA) 	= Not CY8C27xA      This is selected for all other families other than
//														Silicon A of CY8C27x43
//        I2C Clock                	= 100K Standard		Set the I2C clock as 100KHz.
//        I2C Pin		  			= P1[5] - P1[7] 	Select P1[5] and P1[7] for I2C Communication
//
//
//    Note : For further information on the above parameters of I2CHW  
//            see "High Level API section" of I2CHW module datasheet.
//
//	  OneSecTimer
//        Clock						= VC3					The clock source for the user module will be VC3 (10KHz)
//		  Capture					= HIGH					Set to HIGH to disable the software capture
//		  TerminalCountOut			= None					Terminal Count Output is not routed to anywhere
//		  CompareOut				= None					Compare Output is not routed to anywhere
//		  Period					= 9999					Divides the clock signal by value + 1
//		  CompareValue				= 5000					When count value is less than this value compare out will be set to HIGH
//		  CompareType				= Less Than Or Equal	Timer counts will be compared with compare value using this logical operation
//		  Interrupt Type			= Terminal Count		Interrupt will be generated on Terminal Count
//		  ClockSync					= Sync to SysClk		Synchronize the timer clock with SysClk
//		  TC_PulseWidth				= Full Clock			Terminbal Count will stay HIGH for one complete clock width
//		  Invert Capture			= Normal				Makes the enable as active low or active high using this option
//
//	  LCD
//		  LCD Port					= Port2					Connects the LCD at Port2
//		  BarGraph					= Disable				Disables the BarGraph feature.
//
//I2C Lines
//    	Pin	    Select	            Drive
//    	----------------------------------------------
//    	P1[5]	   I2C SDA	    	Open Drain Low
//		P1[7]	   I2C SCL			Open Drain Low
//LCD Lines
//    	Pin	    Select	            Drive
//    	----------------------------------------------
//    	P2[0]	   StdCPU	    	Strong
//		P2[1]	   StdCPU	    	Strong
//    	P2[2]	   StdCPU	    	Strong
//		P2[3]	   StdCPU	    	Strong
//    	P2[4]	   StdCPU	    	Strong
//		P2[5]	   StdCPU	    	Strong
//    	P2[6]	   StdCPU	    	Strong
//
//----------------------------------------------------------------------------

#include <m8c.h>        // Part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "stdlib.h"		// Conversion functions like float to ascii

// Declares the interrupt handler for the timer interrupt 
#pragma interrupt_handler OneSecTimer_ISR

// Flag register used to store and indicate various events
BYTE bReadTempFlag;

// Data after reading back will be stored in this variable
int iTemp;

void ReadTemp(void);
void UpdateDisplay(void);

void main(void)
{
	// Enable the I2CHW module in Master Configuration
    I2CHW_EnableMstr();		
	
	// Start the 16 bit timer
	OneSecTimer_Start();	
	
	// Start the LCD
	LCD_Start();
	
	// Enable the I2C Interrupt
	I2CHW_EnableInt();
	
	// Enable the Timer Interrupt
	OneSecTimer_EnableInt();
	
	// Enable Global Interrupts
	M8C_EnableGInt;
	
	// Set the display pointer at Row:0 and Col:0
	LCD_Position(0,0);
	
	// Display the starting text on the LCD
	LCD_PrCString("Example  Project");
	
	LCD_Position(1,0);
	LCD_PrCString("Curr Temp: ");
	
	// Write data from RAM to EEPROM
	while(1)
	{
		// set upon the timer interrupt
		if(bReadTempFlag)
		{
			// Read the temperature from the temperature sensor
			ReadTemp();
			
			// Process the data and update the LCD
			UpdateDisplay();
			
			// Reset the flag and wait for the next timer interrupt
			bReadTempFlag = 0x00;
		}
	}
}

// Interrupt service routine for 16 bit timer
void OneSecTimer_ISR(void)
{
	// Set this flag on every interrupt event for reading data from the sensor
	bReadTempFlag = 0x01;
}

// This function reads the temperature from the LM75 and stores it in the variable iTempBuffer
void ReadTemp(void)
{
	// Do a read operation to read 2 bytes of data to the iTemp
    I2CHW_fReadBytes(0x48, (char *)&iTemp, 0x02, I2CHW_CompleteXfer);
  	
    // Wait till Read operation is complete and then clear the Read Complete flag
    while (!(I2CHW_bReadI2CStatus() & I2CHW_RD_COMPLETE));
	I2CHW_ClrRdStatus();

    // At this point the data is available in iTemp
}

// This function processes the data read from the sensor and displays it on the LCD
void UpdateDisplay(void)
{
	// Variables used to convert the Hex Temperature data into Decimal format
	float fTemperature;
	char *floatRet;
	int iStatus;
	
	// Shift right by 7 bits because they contain garbage. 
	iTemp = iTemp >> 7;
	
	// Convert the number into floating value by multiplying it with the resolution of the LSB
	fTemperature = (float)iTemp * 0.5;

	// Convert the float to Ascii in order to display it on the LCD
	floatRet = ftoa(fTemperature, &iStatus);

	// Display the data on the LCD
	LCD_Position(1,11);
	LCD_PrString(floatRet);
}